/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
q3master@hendriklipka.de
*/

package de.hendriklipka.q3master;

import org.apache.log4j.*;

import java.net.*;
import java.io.IOException;
import java.io.InterruptedIOException;

public class BroadcastQuery extends Thread
{
    private static Category CAT=Category.getInstance(BroadcastQuery.class.getName());
    private static Category LOG_CAT=Category.getInstance("q3master.broadcast");

    protected String[] _addresses=null;
    protected int _sleepTime=0;

    protected ServerStateManager _handler=null;

    protected DatagramSocket _socket=null;

    public BroadcastQuery(String[] addresses, int sleepTime, ServerStateManager handler)
    {
        super("BroadcastQuery");
        _addresses=addresses;
        _sleepTime=sleepTime;
        _handler=handler;
    }

    public void run()
    {
        _socket=de.hendriklipka.util.UDPSocketPool.getSocket();
        new BroadcastListener(false).start();
        while (true)
        {
            for (int i=0;i<_addresses.length;i++)
            {
                CAT.debug("sending broadcast packet to "+_addresses[i]);
                try
                {
                    Util.sendRequest(_addresses[i],"getstatus".getBytes(),_socket,27960);
                }
                catch (IOException ex)
                {
                    CAT.error(ex);
                }
            }
            Object sleepObj=new Object();
            synchronized (sleepObj)
            {
                try
                {
                    sleepObj.wait(_sleepTime);
                }
                catch (InterruptedException ex) {}
            }
        }
    }

    public void startSingleQuery()
    {
        _socket=de.hendriklipka.util.UDPSocketPool.getSocket();
        try
        {
            _socket.setSoTimeout(1000);
        }
        catch (SocketException e)
        {
            CAT.error(e);
        }
        BroadcastListener bcl=new BroadcastListener(true);
        bcl.start();
        for (int i=0;i<_addresses.length;i++)
        {
            CAT.debug("sending broadcast packet to "+_addresses[i]);
            try
            {
                Util.sendRequest(_addresses[i],"getstatus".getBytes(),_socket,27960);
            }
            catch (IOException ex)
            {
                CAT.error(ex);
            }
        }
        Object sleepObj=new Object();
        synchronized (sleepObj)
        {
            try
            {
                sleepObj.wait(10*1000);
            }
            catch (InterruptedException ex) {}
        }
        bcl.fini=true;
    }

    protected class BroadcastListener extends Thread
    {
        boolean _reportOnly=false;
        public boolean fini=false;

        public BroadcastListener(boolean reportOnly)
        {
            super("BroadCastListener");
            _reportOnly=reportOnly;
        }

        public void run()
        {
            while (!fini)
            {
                try
                {
                    byte[] buffer=new byte[65507]; // maximum packet size
                    DatagramPacket pr=new DatagramPacket(buffer,buffer.length);
                    _socket.receive(pr);
                    InetAddress from=pr.getAddress();
                    int fromPort=pr.getPort();
                    LOG_CAT.info("received broadcast answer from "+from+":"+fromPort);
                    ServerData sd=new ServerData();
                    sd._serverAddress=from;
                    sd._serverPort=fromPort;
                    sd._lastHeartBeat=System.currentTimeMillis();
                    if (_reportOnly)
                    {
                        System.out.println("server found at "+from+":"+fromPort);
                    }
                    else
                    {
                        _handler.addServerByBroadcast(sd);
                    }
                }
                catch (InterruptedIOException ex)
                {
                }
                catch (Exception ex)
                {
                    CAT.error(ex);
                }
            }
        }
    }

}