/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
q3master@hendriklipka.de
*/

package de.hendriklipka.q3master.handler.serverquery;

import org.apache.log4j.*;
import de.hendriklipka.q3master.ProtocolVersionHandler;
import de.hendriklipka.q3master.ServerData;

public class Protocol43Handler implements ProtocolVersionHandler
{
    private Category CAT=Category.getInstance(Protocol43Handler.class.getName());
    private Category LOG_CAT=Category.getInstance("q3master.protocol");

    protected final static byte[] RSP_HDR={-1,-1,-1,-1,'g','e','t','s','e','r','v','e','r','s',
                                           'R','e','s','p','o','n','s','e'};
    protected final static byte[] RSP_EOT={92,0};

    public byte[][] createResponse(ServerData[] servers)
    {
        byte[][] responses=new byte[servers.length][];

        for (int serverNum=0;serverNum<servers.length;serverNum++)
        {
            responses[serverNum]=createResponse(servers[serverNum]);
        }
        return responses;
    }

    protected byte[] createResponse(ServerData server)
    {
        int responseLength=7; // 2 bytes port, 4 bytes address, 1 byte '\\'
        byte[] response=new byte[responseLength];
        response[0]=92; // '\\' as end-of-record marker
        response[1]=server._serverAddress.getAddress()[0];
        response[2]=server._serverAddress.getAddress()[1];
        response[3]=server._serverAddress.getAddress()[2];
        response[4]=server._serverAddress.getAddress()[3];
        response[5]=(byte)(server._serverPort>>8); // high byte of port
        response[6]=(byte)(server._serverPort&0xff); // low byte of port
        return response;
    }

    public byte[] createResponseHeader()
    {
        return RSP_HDR;
    }

    public byte[] createResponseFooter()
    {
        return RSP_EOT;
    }

    public int getBytesPerBlock()
    {
        return 7;
    }

    public int getMaxBlocksPerResponse()
    {
        return 100;
    }

}