package de.hendriklipka.q3master.udpproxy;

import jcmdline.StringParam;
import jcmdline.IntParam;
import jcmdline.CmdLineHandler;
import jcmdline.VersionCmdLineHandler;
import jcmdline.HelpCmdLineHandler;
import jcmdline.Parameter;
import jcmdline.PosixCmdLineParser;

import java.util.Vector;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.io.IOException;

import org.apache.log4j.*;

public class UDPProxy
{
    private static final Category CAT=Category.getInstance("main");
    private static final Category CAT_DBG=Category.getInstance(UDPProxy.class);

    private static final String VERSION="1.0";
    private UDPProxyThread[] _threads=null;

    public static void main(String[] args)
    {
        try
        {
            BasicConfigurator.configure();
            Appender app=new FileAppender(new TTCCLayout(),"udpproxy.log");
            BasicConfigurator.configure(app);
            readParameters(args);
        }
        catch (IOException e)
        {
            e.printStackTrace();  //To change body of catch statement use Options | File Templates.
        }
    }

    private static void readParameters(String[] args)
    {
        String helpText="help";

        StringParam targetIPParam=new StringParam("targetIP","IP Address of the target server",false);
        StringParam localIPParam=new StringParam("localIP", "local IP Address to bind to", true);
        IntParam portParam=new IntParam("port","port to proxy",1,65535,true);
        StringParam portsParam=new StringParam("ports", "port range to proxy", true);

        CmdLineHandler cl=
                new VersionCmdLineHandler("V "+VERSION,
                                          new HelpCmdLineHandler(helpText,
                                                                 "udpproxy",
                                                                 "proxy one or multiple UDP ports to a target server, generating a debug log",
                                                                 new Parameter[]{targetIPParam, localIPParam, portParam, portsParam},
                                                                 new Parameter[]{},
                                                                 new PosixCmdLineParser()));
        cl.parse(args);

        String targetAddress=null;
        String localAddress=null;
        int[] ports=null;

        if (localIPParam.isSet())
        {
            localAddress=localIPParam.getValue();
        }
        targetAddress=targetIPParam.getValue();
        if ((portParam.isSet()&&portsParam.isSet())||(!portParam.isSet()&&!portsParam.isSet()))
        {
            cl.exitUsageError("exactly one of the options '-port' and '-ports' must be given");
        }
        if (portParam.isSet())
        {
            ports=new int[1];
            ports[0]=Integer.parseInt(portParam.getValue());
        }
        else
        {
            ports=readPortsParameter(portsParam.getValue(),cl);
        }
        try
        {
            UDPProxy proxy=new UDPProxy(targetAddress, localAddress, ports);
            proxy.run();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    private static int[] readPortsParameter(String ports, CmdLineHandler cl)
    {
        Vector v=new Vector();
        while (true)
        {
            int pos=ports.indexOf(':');
            String currentPort=ports;
            if (-1!=pos)
            {
                currentPort=ports.substring(0, pos);
                ports=ports.substring(pos+1);
            }
            try
            {
                int pos2=currentPort.indexOf('-');
                if (-1==pos2)
                {
                    int port=Integer.parseInt(currentPort);
                    if (0==port)
                        cl.exitUsageError("invalid port: 0 is not allowed!");
                    v.addElement(new Integer(port));
                }
                else if (0==pos2)
                {
                    currentPort=currentPort.substring(1);
                    int port=Integer.parseInt(currentPort);
                    if (0==port)
                        cl.exitUsageError("invalid port: 0 is not allowed!");
                    if (port>65535)
                        cl.exitUsageError("invalid port: must be lower than 65536!");
                    for (int i=1; i<=port; i++)
                        v.addElement(new Integer(i));
                }
                else if ((currentPort.length()-1)==pos2)
                {
                    v.addElement(currentPort);
                }
                else
                {
                    String firstPortStr=currentPort.substring(0, pos2);
                    String lastPortStr=currentPort.substring(pos2+1);
                    int firstPort=Integer.parseInt(firstPortStr);
                    int lastPort=Integer.parseInt(lastPortStr);
                    if (lastPort<=firstPort)
                        cl.exitUsageError("ports: range must be in correct order!");
                    if (0==firstPort)
                        cl.exitUsageError("invalid port: 0 is not allowed!");
                    if (lastPort>65535)
                        cl.exitUsageError("invalid port: must be lower than 65536!");
                    for (int i=firstPort; i<=lastPort; i++)
                        v.addElement(new Integer(i));
                }
                if (-1==pos)
                    break;
            }
            catch (NumberFormatException e)
            {
                cl.exitUsageError("ports: invalid number given ("+currentPort+")");
            }
        }

        int[] result=new int[v.size()];
        for (int i=0; i<v.size(); i++)
        {
            Integer port=(Integer)v.elementAt(i);
            result[i]=port.intValue();
        }
        return result;
    }

    public UDPProxy(String targetAddress, String localAddress, int[] ports) throws SocketException, UnknownHostException
    {
        _threads=new UDPProxyThread[ports.length];
        for (int i=0; i<ports.length; i++)
        {
            _threads[i]=new UDPProxyThread(targetAddress,localAddress, ports[i]);
        }
    }

    protected void run()
    {
        Runtime.getRuntime().addShutdownHook(new ShutdownThread());
        for (int i=0; i<_threads.length; i++)
        {
            UDPProxyThread thread=_threads[i];
            thread.start();
        }
        synchronized(this)
        {
            try
            {
                wait();
            }
            catch (InterruptedException ex) {}
        }
    }

    private class ShutdownThread extends Thread
    {
        public void run()
        {
            CAT_DBG.debug("killing all threads");
            for (int i=0; i<_threads.length; i++)
            {
                UDPProxyThread thread=_threads[i];
                thread.stopThread();
                try
                {
                    thread.join(2000);
                }
                catch (InterruptedException ex) {}
            }
        }
    }
}
